package bdd.search.query;

import bdd.search.EnginePrefs;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;

/** Written by Tim Macinta 1997                           <br>
 *  Distributed under the GNU Public License
 *       (a copy of which is enclosed with the source).   <br>
 *                                                        <br> 
 *  This class searches the database for the given query.
 */
public class DBQuery {
  String query;
  EnginePrefs prefs;

  public DBQuery(String query, EnginePrefs prefs) {
    this.query = query;
    this.prefs = prefs;
  }

  public void dumpResults(OutputStream stream) throws IOException {
    
    // create a DataOutputStream

    DataOutputStream out;
    if (stream instanceof DataOutputStream) {
      out = (DataOutputStream) stream;
    } else {
      out = new DataOutputStream(stream);
    }
    
    // determine results of query

    StringTokenizer st = new StringTokenizer(makeAlphaNumeric(query.toLowerCase()), " ");
    Vector results = new Vector(10, 20);
    while (st.hasMoreTokens()) {
      try {
	addResultsToVector(st.nextToken(), results);
      } catch (IOException e) {}
    }

    // handle the case where no results are found

    File f;
    if (results.size() < 1) {
      f = prefs.getNotFoundFile();
      if (f.exists()) {
	pipe(new FileInputStream(f), stream);
      }
      return;
    }

    // output header file
    
    f = prefs.getHeaderFile();
    if (f.exists()) {
      pipe(new FileInputStream(f), stream);
    }

    // output query results

    Enumeration en = results.elements();
    int si = results.size();
    if (si == 1) {
      out.writeBytes("<pre>\n<bold>There was 1 result:\n\n");
    } else {
      out.writeBytes("<pre>\n<bold>There were "+si+" results:\n\n");
    }
    while (en.hasMoreElements()) {
      out.writeBytes("   "+((String) en.nextElement()) + "<br>\n");
    }
    out.writeBytes("</pre>\n");

    // output footer file
    
    f = prefs.getFooterFile();
    if (f.exists()) {
      pipe(new FileInputStream(f), stream);
    }

  }

  /** Returns a copy of "word" where all non-alphanumeric characters are
   *  converted to spaces (ascii 32).
   */
  String makeAlphaNumeric(String word) {
    char[] w = word.toCharArray();
    for (int i = w.length - 1; i >= 0; i--) {
      if (!Character.isLetterOrDigit(w[i])) w[i] = ' ';
    }
    return new String(w);
  }

  /** Searches for the word contained in the database and adds a hyperlinked
   *  String describing each match to the Vector "v" with the most relevant
   *  matches being added first.
   */
  void addResultsToVector(String the_query, Vector v) throws IOException {
    DataInputStream in = new DataInputStream(new FileInputStream(prefs.getMainIndex()));
    in.skip(in.readLong()-8);
    
    // find the word

    String word = in.readLine();
    while (word != null && word.compareTo(the_query) < 0) {
      while (in.readInt() != 0) in.read();
      word = in.readLine();
    }
    if (word != null && word.equals(the_query)) {
      
      // word found

      Vector scores = new Vector(5, 10);
      Vector numbers = new Vector(5, 10);
      Hashtable num = new Hashtable(8);
      Integer i = new Integer(in.readInt());
      while (i.intValue() != 0) {
	numbers.addElement(i);
	num.put(i, i);
	scores.addElement(new Integer(((int)in.read()) & 0xff));
	i = new Integer(in.readInt());
      }

      // load URL descriptions

      in.close();
      in = new DataInputStream(new FileInputStream(prefs.getMainIndex()));
      in.readLong();
      int targ = in.readInt();
      for (int i2 = 1; i2 <= targ; i2++) {
	i = new Integer(i2);
	if (num.get(i) != null) {
	  num.put(i, in.readLine().trim());
	} else {
	  in.readLine();
	}
      }

      // add URL descriptions to "v" in proper order

      Enumeration en1 = scores.elements();
      Enumeration en2 = numbers.elements();
      int i3;
      String result;
      String url;
      while (en1.hasMoreElements()) {

	// convert score from 0-255 to 0-100

	i3 = ((Integer) en1.nextElement()).intValue();
	i3 = (i3 * 100)/255;
	if (i3 > 99) {
	  result = "";
	} else if (i3 > 9) {
	  result = " ";
	} else {
	  result = "  ";
	}
	url = (String) (num.get(en2.nextElement()));
	result = result + i3 + "  -  <a href=\""+url+"\">"+url+"</a>";
	v.addElement(result);
      }
    }
    in.close();
  }

  /** This is the method that is called when this class is invoked from the
   *  command line.  If this method is called, the first argument (arg[0])
   *  is used as a new query and the results are piped to standard output.
   *  This could be used as a means for providing a cgi interface to the
   *  search database, although it isn't as efficient as running the
   *  custom web server that was built for this since running this via
   *  cgi requires starting a new copy of the Java Virtual Machine each
   *  time it is run.
   */
  public static void main(String arg[]) {
    try {
      String query;
      if (arg.length < 1) {
	query = "";
      } else {
	query = arg[0];
      }
      DBQuery dbq = new DBQuery(query, new EnginePrefs());
      dbq.dumpResults(System.out);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  /** Pipes "in" to "out" until "in" is exhausted then closes "in".   */
  void pipe(InputStream in, OutputStream out) throws IOException {
    byte[] b = new byte[512];
    int x = in.read(b, 0, b.length);
    while (x > 0) {
      out.write(b, 0, x);
      x = in.read(b, 0, b.length);
    }
    in.close();
  }

}
