package bdd.search.query;

import java.net.ServerSocket;
import java.io.IOException;
import bdd.search.EnginePrefs;

/** Written by Tim Macinta 1997                           <br>
 *  Distributed under the GNU Public License
 *       (a copy of which is enclosed with the source).   <br>
 *                                                        <br> 
 *  This class provides a specialized web server that allows queries
 *  into a search database.
 */
public class QueryWebServer extends Thread {

  int port = EnginePrefs.port;  // the TCP port number for this server
  EnginePrefs prefs;  // preferences

  /** Starts a new QueryWebServer on the default port.  */
  public QueryWebServer() {
    this(EnginePrefs.port, new EnginePrefs());
  }

  /** Starts a new QueryWebServer on the specified port.  */
  public QueryWebServer(int port) {
    this(port, new EnginePrefs());
  }

  /** Starts a new QueryWebServer on the specified port with the given
   *  preferences.  */
  public QueryWebServer(int port, EnginePrefs prefs) {
    this.prefs = prefs;
    this.port = port;
    start();
  }

  /** This is where connections are accepted.  */
  public void run() {
    try {
      ServerSocket sock = new ServerSocket(port);
      while (true) {
	try {
	  new QueryConnection(sock.accept(), prefs);
	} catch (IOException e) {}
      }
    } catch (IOException e2) {
      e2.printStackTrace();
      System.err.println("\n\nWarning:  could not bind to port "+port);
      System.err.println("The web server won't work now.\n");
    }
  }

  /** This is the method that is called when the server is started from
   *  the command line.
   */
  public static void main(String arg[]) {
    new QueryWebServer();
  }

}
